﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;

using SlimDX.Direct3D11;
using SlimDX;


using VVVV.PluginInterfaces.V2;
using VVVV.PluginInterfaces.V1;

using FeralTic.Resources.Geometry;
using FeralTic.DX11.Resources;
using FeralTic.DX11;
using FeralTic.DX11.Geometry;

namespace VVVV.DX11.ANT
{
    [PluginInfo(Name = "GridWarp", Category = "DX11.Geometry", Version = "", Author = "antokhio")]
    public class GridWarp : IPluginEvaluate, IDX11ResourceHost
    {

        [Input("fx1")]
        IDiffSpread<float> fx1;

        [Input("fx2")]
        IDiffSpread<float> fx2;

        [Input("fy1")]
        IDiffSpread<float> fy1;

        [Input("fy2")]
        IDiffSpread<float> fy2;

        [Input("scale")]
        IDiffSpread<Vector2> fScale;

        [Input("scale2")]
        IDiffSpread<Vector2> fScaleY;

        [Output("Geometry")]
        protected ISpread<DX11Resource<DX11IndexedGeometry>> geometryOutput;

        private bool invalidate = false;



        public void Evaluate(int SpreadMax)
        {
  
            if (geometryOutput[0] == null)
            {
                geometryOutput[0] = new DX11Resource<DX11IndexedGeometry>();
            }

            if (fx1.IsChanged || fx2.IsChanged || fy1.IsChanged || fy2.IsChanged || fScale.IsChanged || fScaleY.IsChanged)
            {
                geometryOutput[0].Dispose();
                geometryOutput[0] = new DX11Resource<DX11IndexedGeometry>();
            }
        }

/*
-0.5000 0.5000
-0.1667 0.5000
0.1667 0.5000
0.5000 0.5000
-0.5000 0.1667
-0.1667 0.1667
0.1667 0.1667
0.5000 0.1667
-0.5000 -0.1667
-0.1667 -0.1667
0.1667 -0.1667
0.5000 -0.1667
-0.5000 -0.5000
-0.1667 -0.5000
0.1667 -0.5000
0.5000 -0.5000
*/



        public void Update(DX11RenderContext context)
        {
            //if (this.invalidate || !geometryOutput[0].Contains(context))
            if (!geometryOutput[0].Contains(context))
                {

                Vertex[] points = new Vertex[16];

                points[0] = new Vertex(new Vector2(-0.5000f + fScale[0].X, 0.5000f + fScaleY[0].X), new Vector2(0.0000f, 0.0000f));
                points[1] = new Vertex(new Vector2((-0.1667f + fx1[0]) + fScale[0].X, 0.5000f + fScaleY[0].X), new Vector2(0.3333f + fx1[0], 0.0000f));
                points[2] = new Vertex(new Vector2((0.1667f  + fx2[0]) + fScale[0].Y, 0.5000f + fScaleY[0].X), new Vector2(0.6667f + fx2[0], 0.0000f));
                points[3] = new Vertex(new Vector2(0.5000f + fScale[0].Y, 0.5000f + fScaleY[0].X), new Vector2(1.0000f, 0.0000f));

                points[4] = new Vertex(new Vector2(-0.5000f + fScale[0].X, 0.1667f + fy1[0] + fScaleY[0].X), new Vector2(0.0000f, 0.3333f - fy1[0]));
                points[5] = new Vertex(new Vector2((-0.1667f + fx1[0]) + fScale[0].X, 0.1667f + fy1[0] + fScaleY[0].X), new Vector2(0.3333f + fx1[0], 0.3333f - fy1[0]));
                points[6] = new Vertex(new Vector2((0.1667f + fx2[0]) + fScale[0].Y, 0.1667f + fy1[0] + fScaleY[0].X), new Vector2(0.6667f + fx2[0], 0.3333f - fy1[0]));
                points[7] = new Vertex(new Vector2(0.5000f + fScale[0].Y, 0.1667f + fy1[0] + fScaleY[0].X), new Vector2(1.0000f, 0.3333f - fy1[0]));

                points[8] = new Vertex(new Vector2(-0.5000f + fScale[0].X, -0.1667f + fy2[0] + fScaleY[0].Y), new Vector2(0.0000f, 0.6667f - fy2[0]));
                points[9] = new Vertex(new Vector2((-0.1667f + fx1[0]) + fScale[0].X, -0.1667f + fy2[0] + fScaleY[0].Y), new Vector2(0.3333f + fx1[0], 0.6667f - fy2[0]));
                points[10] = new Vertex(new Vector2((0.1667f + fx2[0]) + fScale[0].Y, -0.1667f + fy2[0] + fScaleY[0].Y), new Vector2(0.6667f + fx2[0], 0.6667f - fy2[0]));
                points[11] = new Vertex(new Vector2(0.5000f + fScale[0].Y, -0.1667f + fy2[0] + fScaleY[0].Y), new Vector2(1.0000f, 0.6667f - fy2[0]));

                points[12] = new Vertex(new Vector2(-0.5000f + fScale[0].X, -0.5000f + fScaleY[0].Y), new Vector2(0.0000f, 1.0000f));
                points[13] = new Vertex(new Vector2((-0.1667f + fx1[0]) + fScale[0].X, -0.5000f + fScaleY[0].Y), new Vector2(0.3333f + fx1[0], 1.0000f));
                points[14] = new Vertex(new Vector2((0.1667f + fx2[0]) + fScale[0].Y, -0.5000f + fScaleY[0].Y), new Vector2(0.6667f + fx2[0], 1.0000f));
                points[15] = new Vertex(new Vector2(0.5000f + fScale[0].Y, -0.5000f + fScaleY[0].Y), new Vector2(1.0000f, 1.0000f));


               

                int[] indecies = new int[54];

                indecies[0] = 0; indecies[1] = 1; indecies[2] = 4;
                indecies[3] = 4; indecies[4] = 1; indecies[5] = 5;

                indecies[6] = 1; indecies[7] = 2; indecies[8] = 5;
                indecies[9] = 5; indecies[10] = 2; indecies[11] = 6;

                indecies[12] = 2; indecies[13] = 3; indecies[14] = 6;
                indecies[15] = 6; indecies[16] = 3; indecies[17] = 7;

                indecies[18] = 4; indecies[19] = 5; indecies[20] = 8;
                indecies[21] = 8; indecies[22] = 5; indecies[23] = 9;

                indecies[24] = 5; indecies[25] = 6; indecies[26] = 9;
                indecies[27] = 9; indecies[28] = 6; indecies[29] = 10;

                indecies[30] = 6; indecies[31] = 7; indecies[32] = 10;
                indecies[33] = 10; indecies[34] = 7; indecies[35] = 11;

                indecies[36] = 8; indecies[37] = 9; indecies[38] = 12;
                indecies[39] = 12; indecies[40] = 9; indecies[41] = 13;

                indecies[42] = 9; indecies[43] = 10; indecies[44] = 13;
                indecies[45] = 13; indecies[46] = 10; indecies[47] = 14;

                indecies[48] = 10; indecies[49] = 11; indecies[50] = 14;
                indecies[51] = 14; indecies[52] = 11; indecies[53] = 15;

                DX11IndexedGeometry ig = DX11IndexedGeometry.CreateFrom<Vertex>(context, points, indecies, Pos2Tex2Vertex.Layout);
                ig.Topology = SlimDX.Direct3D11.PrimitiveTopology.TriangleList;

                geometryOutput[0][context] = ig;
            }
        }
        public void Destroy(DX11RenderContext context, bool force)
        {
            if (geometryOutput[0] != null)
            {
                geometryOutput[0].Dispose(context);
            }
        }
    }

    public struct Vertex
    {
        public Vector2 pos, uv;
        public Vertex(Vector2 p, Vector2 u)
        {
            pos = p;
            uv = u;
        }

    }
}
